from pylab import *
import csv
import networkx as nx

def readFile(filename) :
    """reads a text file, where each line row is a pair of vertices
    connected by an edge. Returns a networkX graph of the described
    graph.

    """ 
    G = nx.read_edgelist(filename)
    return G

def localClusteringCoeff(G) :
    """returns the mean local clustering coefficient for the graph G

    """
    return nx.average_clustering(G)

def allDegrees(G) :
    """for a graph G of size N, this function returns a list of N
    integers, representing the degrees of all of the nodes in the
    graph

    """
    return [d for d in G.degree().values()]

def randomGraphFromDegreeDistribution(G,visualize=False) :
    """takes a graph G as an argument, and returns a random graph with a
    similar size and degree distribution

    """
    G = G.copy()
    H = G.copy()
    H = nx.configuration_model([x for x in G.degree().values()],create_using=H)

    if visualize :
        ## generate a distribution from the degrees of G
        degrees = [G.degree(n) for n in G.nodes()]
        bins = range(0,25)
        deg_dist,histogram_bins = histogram(degrees,bins)

        ## normalize the distribution
        deg_dist = array(deg_dist,dtype=np.float)
        deg_dist_norm = deg_dist / sum(deg_dist)
        plot(histogram_bins[:-1],deg_dist,'kx-',label='original degrees')
        
        ys,histogram_bins = histogram([x for x in H.degree().values()],bins)
        ys = array(ys,dtype=np.float)
        print(sum(ys))
        print(sum(deg_dist))
        ps = ys / sum(ys)
        plot(histogram_bins[:-1],ys,'b.-',label='deg(H)')
        legend()

    return H

def plotGraph(G) :
    ################################### DRAW NETWORK AND DEGREE DISTRIBUTION
    ## set up plots
    fig = plt.figure(figsize=(6,15))
    plotA = subplot2grid((2,1),(0,0))
    plotB = subplot2grid((2,1),(1,0))

    ## DRAW NETWORK
    #pos = nx.spring_layout(G,iterations=1000,k=0.05)
    pos = nx.circular_layout(G)
    nodes = nx.draw_networkx_nodes(G,pos,node_size=10,ax=plotA,alpha=0.5)
    edges = nx.draw_networkx_edges(G,pos,ax=plotA,alpha=0.1)
    plotA.set_title('')

    ## plot actual degree distribution of graph
    degrees = [G.degree(n) for n in G.nodes()]
    ys,xs = histogram(degrees,range(1,25))
    plotB.plot(xs[:-1],ys,'kx-',label='graph degree distribution')



